
#pragma once

#include "CoreMinimal.h"
#include "EffekseerRendererNative.h"

#include <EffekseerEffect.h>
#include <map>
#include <memory>

namespace EffekseerRendererUE4
{
	class RendererImplemented;
	class RenderState;
	class VertexBuffer;
	class IndexBuffer;
	class Shader;

	using Vertex = EffekseerRenderer::SimpleVertex;
	using VertexDistortion = EffekseerRenderer::VertexDistortion;

	typedef ::Effekseer::ModelRenderer::NodeParameter efkModelNodeParam;
	typedef ::Effekseer::ModelRenderer::InstanceParameter efkModelInstanceParam;
	typedef ::Effekseer::Vector3D efkVector3D;

	class ModelRenderer
		: public ::EffekseerRenderer::ModelRendererBase
	{
	private:
		RendererImplemented*	m_renderer;
		ModelRenderer(RendererImplemented* renderer);

	public:

		virtual ~ModelRenderer();

		static ModelRenderer* Create(RendererImplemented* renderer);

	public:
		void BeginRendering(const efkModelNodeParam& parameter, int32_t count, void* userData);

		void Rendering(const efkModelNodeParam& parameter, const efkModelInstanceParam& instanceParameter, void* userData) override;

		void EndRendering(const efkModelNodeParam& parameter, void* userData);
	};

	class RendererImplemented
		: public ::EffekseerRenderer::Renderer
		, public ::Effekseer::ReferenceObject
	{
	protected:
		::Effekseer::Vector3D	m_lightDirection;
		::Effekseer::Color		m_lightColor;
		::Effekseer::Color		m_lightAmbient;
		int32_t					m_squareMaxCount;

		VertexBuffer*			m_vertexBuffer = nullptr;

		std::unique_ptr<Shader> stanShader_;
		std::unique_ptr<Shader> backDistortedShader_;
		std::unique_ptr<Shader> lightingShader_;

		Shader*					m_currentShader = nullptr;
		RenderState*			m_renderState = nullptr;

		std::array<void*, Effekseer::TextureSlotMax> textures_;

		FMatrix					m_localToWorld;
		int32_t					m_viewIndex = 0;
		TMap<UTexture2D*, UMaterialInstanceDynamic*>*	m_materials[6];
		FMeshElementCollector*	m_meshElementCollector = nullptr;
		std::map<EffekseerEffectMaterial, UMaterialInstanceDynamic*>	m_nmaterials;

		float m_distortionIntensity = 0.0f;

		EffekseerRenderer::StandardRenderer<RendererImplemented, Shader, Vertex, VertexDistortion>*	m_standardRenderer = nullptr;
	public:
		
		static RendererImplemented* Create();

		RendererImplemented();
		virtual ~RendererImplemented();

		void OnLostDevice() override {}
		void OnResetDevice() override {}

		/**
		@brief	
		*/
		bool Initialize(int32_t squareMaxCount);

		/**
		@brief	̃CX^XjB
		*/
		void Destroy() override;

		/**
		@brief	Xe[g𕜋A邩ǂ̃tOݒ肷B
		*/
		void SetRestorationOfStatesFlag(bool flag) override;

		/**
		@brief	`Jn鎞ɎsB
		*/
		bool BeginRendering() override;

		/**
		@brief	`I鎞ɎsB
		*/
		bool EndRendering() override;

		/**
		@brief	Cg̕擾B
		*/
		::Effekseer::Vector3D GetLightDirection() const override;

		/**
		@brief	Cg̕ݒ肷B
		*/
		void SetLightDirection(const ::Effekseer::Vector3D& direction) override;

		/**
		@brief	Cg̐F擾B
		*/
		const ::Effekseer::Color& GetLightColor() const override;

		/**
		@brief	Cg̐Fݒ肷B
		*/
		void SetLightColor(const ::Effekseer::Color& color) override;

		/**
		@brief	Cg̊̐F擾B
		*/
		const ::Effekseer::Color& GetLightAmbientColor() const override;

		/**
		@brief	Cg̊̐Fݒ肷B
		*/
		void SetLightAmbientColor(const ::Effekseer::Color& color) override;

		/**
		@brief	ő`XvCg擾B
		*/
		int32_t GetSquareMaxCount() const  override;

		/**
		@brief	XvCg_[𐶐B
		*/
		::Effekseer::SpriteRenderer* CreateSpriteRenderer() override;

		/**
		@brief	{_[𐶐B
		*/
		::Effekseer::RibbonRenderer* CreateRibbonRenderer() override;

		/**
		@brief	O_[𐶐B
		*/
		::Effekseer::RingRenderer* CreateRingRenderer() override;

		/**
		@brief	f_[𐶐B
		*/
		::Effekseer::ModelRenderer* CreateModelRenderer() override;

		/**
		@brief	OՃ_[𐶐B
		*/
		::Effekseer::TrackRenderer* CreateTrackRenderer() override;

		virtual ::Effekseer::MaterialLoader* CreateMaterialLoader(::Effekseer::FileInterface* fileInterface = nullptr) override { return nullptr; }
		/**
		@brief	W̃eNX`ǍNX𐶐B
		*/
		::Effekseer::TextureLoader* CreateTextureLoader(::Effekseer::FileInterface* fileInterface = NULL)  override;

		/**
		@brief	W̃fǍNX𐶐B
		*/
		::Effekseer::ModelLoader* CreateModelLoader(::Effekseer::FileInterface* fileInterface = NULL) override;

		/**
		@brief	_[Xe[gIɃZbgB
		*/
		void ResetRenderState() override;

		/**
		@brief	wic܂GtFNg`悳OɌĂ΂R[obN擾B
		*/
		::EffekseerRenderer::DistortingCallback* GetDistortingCallback()  override;

		/**
		@brief	wic܂GtFNg`悳OɌĂ΂R[obNݒ肷B
		*/
		void SetDistortingCallback(::EffekseerRenderer::DistortingCallback* callback) override;

		int32_t GetDrawCallCount() const override { return 0; }

		int32_t GetDrawVertexCount() const override { return 0; }

		void ResetDrawCallCount() override { }

		void ResetDrawVertexCount() override { }

		void SetRenderMode(Effekseer::RenderMode renderMode) override { }

		Effekseer::TextureData* GetBackground();

		VertexBuffer* GetVertexBuffer();

		IndexBuffer* GetIndexBuffer();

		EffekseerRenderer::StandardRenderer<RendererImplemented, Shader, Vertex, VertexDistortion>* GetStandardRenderer();

		::EffekseerRenderer::RenderStateBase* GetRenderState();

		void SetVertexBuffer(VertexBuffer* vertexBuffer, int32_t size);
		void SetIndexBuffer(IndexBuffer* indexBuffer);

		void SetLayout(Shader* shader);
		void DrawSprites(int32_t spriteCount, int32_t vertexOffset);
		
		void DrawModel(void* model, std::vector<Effekseer::Matrix44>& matrixes, std::vector<Effekseer::RectF>& uvs, std::vector<Effekseer::Color>& colors, std::vector<int32_t>& times, std::vector<std::array<float, 4>>& customData1, std::vector<std::array<float, 4>>& customData2);

		UMaterialInterface* FindMaterial();

		Shader* GetShader(bool useTexture, ::Effekseer::RendererMaterialType) const;

		void BeginShader(Shader* shader);
		void EndShader(Shader* shader);

		void SetVertexBufferToShader(const void* data, int32_t size, int32_t dstOffset);

		void SetPixelBufferToShader(const void* data, int32_t size, int32_t dstOffset);

		void SetTextures(Shader* shader, Effekseer::TextureData** textures, int32_t count);
		void SetDistortionIntensity(float value) { m_distortionIntensity = value; }

		void SetLocalToWorld(FMatrix localToWorld);
		void SetViewIndex(int32_t viewIndex);
		void SetMaterials(const TMap<UTexture2D*, UMaterialInstanceDynamic*>* materials, int32_t index);
		void SetNMaterials(const std::map<EffekseerEffectMaterial, UMaterialInstanceDynamic*>& nmaterials);
		void SetMeshElementCollector(FMeshElementCollector* meshElementCollector);

		virtual int GetRef() { return ::Effekseer::ReferenceObject::GetRef(); }
		virtual int AddRef() { return ::Effekseer::ReferenceObject::AddRef(); }
		virtual int Release() { return ::Effekseer::ReferenceObject::Release(); }
	};

}